#!/usr/bin/python
# -*- coding: utf-8 -*-
# Pierre Haessig — September 2015
""" Convert a Matpower power flow case from Excel to a Matlab file

"""

from __future__ import division, print_function, unicode_literals
from openpyxl import load_workbook
from jinja2 import Environment, FileSystemLoader
from itertools import islice
import os.path
import argparse


class MPC(object):
    '''just a namespace class for Matpower case data'''
    pass


def parse_args():
    '''define command line interface and then parse command line arguments
    
    returns parser.parse_args()
    '''
    parser = argparse.ArgumentParser(
        description='Convert a Matpower powerflow case from Excel to a Matlab file')

    parser.add_argument(
        'input',
        help="Excel spreadsheet (xls or xlsx) to convert")
    args = parser.parse_args()
    return args


def read_sheet_vals(ws):
    '''read the lines of the sheet `ws`'''
    rows = ws.iter_rows()
    # drop 3 header lines
    header_lines = 3
    rows =islice(rows, header_lines, None)

    vals = []
    for row in rows:
        vals.append([cell.value for cell in row])
    
    return vals


def read_excel(fname):
    '''read Excel spreadsheet and return MPC instance'''
    print('reading Excel file "{}"...'.format(fname))

    # data_only=True -> read values instead of formulas
    wb = load_workbook(fname, use_iterators=True, data_only=True)
    sheet_names = wb.get_sheet_names()

    # check that all sheets are present in the workbook
    for sheet in ['cover', 'bus', 'branch', 'gen', 'gencost']:
        assert sheet in  sheet_names

    mpc = MPC()

    ### Extract data
    mpc.case_name = wb['cover']['A4'].value
    print('case name: {}'.format(mpc.case_name))

    mpc.baseMVA = float(wb['cover']['B4'].value)

    try:
        mpc.comments = wb['cover']['A7'].value
        print('comment: {}'.format(mpc.comments))
    except IndexError:
        mpc.comments = ''
        print('no comment provided')

    # TODO:??
    #bus_val_types = []

    mpc.bus = read_sheet_vals(wb['bus'])
    print('  {} buses read'.format(len(mpc.bus)))

    mpc.branch = read_sheet_vals(wb['branch'])
    print('  {} branches read'.format(len(mpc.branch)))

    mpc.gen = read_sheet_vals(wb['gen'])
    print('  {} generators read'.format(len(mpc.gen)))

    mpc.gencost = read_sheet_vals(wb['gencost'])
    print('  {} gen cost lines read'.format(len(mpc.gencost)))
    
    return mpc


def write_mpc(mpc):
    '''Write the Matlab case file
    
    filename is mpc.case_name + '.m' extension
    '''
    fname = mpc.case_name + '.m'

    dirname = os.path.dirname(__file__)
    env = Environment(
        loader=FileSystemLoader(dirname),
        trim_blocks=True)

    def items_format(row, fmt='g'):
        'format each item of the list `row`'
        fmt = '{:'+fmt+'}'
        return [fmt.format(c) for c in row]
    env.filters['items_format'] = items_format

    template = env.get_template('case_template.m')

    with open(fname, 'w') as mfile:
        mfile.write(template.render(mpc=mpc))

    print('MATPOWER case written in "{}"'.format(fname))


if __name__ == '__main__':
    fname = parse_args().input
    mpc = read_excel(fname)
    write_mpc(mpc)
